@@ -1,18 +1,18 @@
-/* proxyknife -- customizable proxy hunter
- * Copyright (C) 2005-2006 Jia Wang (skyroam@gmail.com)
+/* fetchproxylist.c - routines used to download proxylist from the web
+ * Copyright (C) 2005-2006 Jia Wang <skyroam@gmail.com>
  *
  *
  *
- * This file is part of proxyknife.
- * Proxyknife is free software; you can redistribute it and/or modify it 
+ * This file is part of GNU Proxyknife.
+ * GNU Proxyknife is free software; you can redistribute it and/or modify it 
  * under the terms of the GNU General Public License as published by the 
  * Free Software Foundation; either version 2 of the License, or (at your 
  * option) any later version.
  *
- * Proxyknife is distributed in the hope that it will be useful, but WITHOUT ANY 
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or 
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
- * for more details.
+ * GNU Proxyknife is distributed in the hope that it will be useful, 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ * General Public License for more details.
  *
  *
  * You should have received a copy of the GNU General Public License 
@@ -110,19 +110,6 @@ fetchproxylist (char *proxyknife_in)
   if (debug)
     printf ("Connect to host %s to fetch %s\n", host, path);
 
-  h = gethostbyname (host);
-
-  if (h == NULL)
-    {
-      herror ("gethostbyname");
-      xexit (EXIT_FAILURE);
-    }
-
-  memset (&addr.sin_zero, 0, sizeof (addr.sin_zero));
-  addr.sin_family = AF_INET;
-  addr.sin_port = htons (80);
-  addr.sin_addr = *(struct in_addr *) h->h_addr;
-
   sockfd = socket (AF_INET, SOCK_STREAM, 0);
   if (sockfd == -1)
     {
@@ -130,45 +117,7 @@ fetchproxylist (char *proxyknife_in)
       xexit (EXIT_FAILURE);
     }
 
-  if (my.mytype == DIRECT)
-    {
-      /* here need to add myproxy support */
-
-      n =
-	connect (sockfd, (struct sockaddr *) &addr, sizeof (struct sockaddr));
-      if (n == -1)
-	{
-	  perror (__FILE__ "connect");
-	  xexit (EXIT_FAILURE);
-	}
-    }
-  else if ((my.mytype == HTTP_CONNECT) || (my.mytype == HTTP_CONNECT_AUTH))
-    {
-
-      n = CONVIAHTTP_MAIN (sockfd, host, "80");
-      if (n == -1)
-	{
-	  fprintf (stderr, __FILE__ "fetchproxylist: CONNVIAHTTP_MAIN");
-	  xexit (EXIT_FAILURE);
-	}
-    }
-  else if ((my.mytype == SOCKS5_CONNECT)
-	   || (my.mytype == SOCKS5_CONNECT_AUTH))
-    {
-      n = CONVIASOCKS5_MAIN (sockfd, (struct sockaddr *) (&addr));
-      if (n == -1)
-	{
-	  fprintf (stderr, __FILE__ "fetchproxylist: CONVIASOCKS5_MAIN");
-	  xexit (EXIT_FAILURE);
-	}
-    }
-  else
-    {
-      fprintf (stderr,
-	       __FILE__ "fetchproxylist: mytype=%d is not supported!\n",
-	       my.mytype);
-      xexit (EXIT_FAILURE);
-    }
+  MY_CONNECT (sockfd, host, "80");
 
   /* build http request */
   n = strlen ("GET " " HTTP/1.0" "\r\n" "Host: " "\r\n" "\r\n");
@@ -348,7 +297,7 @@ sgetaline (char *s, struct thread_mem *thread_mem)
     }
   p++;
   len = p - s;
-  thread_mem->line = pxmalloc (len + 1, thread_mem);
+  thread_mem->line = pmalloc (len + 1, thread_mem);
 
   if (thread_mem->line == NULL)
     {
