@@ -1,18 +1,18 @@
 /* readconf.c - routines used to read configuration file and set global vars
- * Copyright (C) 2005-2006 Jia Wang (skyroam@gmail.com)
+ * Copyright (C) 2005-2006 Jia Wang <skyroam@gmail.com>
  *
  *
  *
- * This file is part of proxyknife.
- * Proxyknife is free software; you can redistribute it and/or modify it 
+ * This file is part of GNU Proxyknife.
+ * GNU Proxyknife is free software; you can redistribute it and/or modify it 
  * under the terms of the GNU General Public License as published by the 
  * Free Software Foundation; either version 2 of the License, or (at your 
  * option) any later version.
  *
- * Proxyknife is distributed in the hope that it will be useful, but WITHOUT ANY 
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or 
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
- * for more details.
+ * GNU Proxyknife is distributed in the hope that it will be useful, 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ * General Public License for more details.
  *
  *
  * You should have received a copy of the GNU General Public License 
@@ -27,6 +27,7 @@ int readint (char *line, char *flagstring, int *i, int *num);
 int readstr (char *line, char *flagstring, char **s, int *num);
 void optstr (char *value, char **s, int *num);
 void optint (char *value, int *i, int *num);
+void * xstrdup(char *s);
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,32 +40,37 @@ extern char *optarg;
 extern int optind, opterr, optopt;
 #include <getopt.h>
 
+#include <assert.h>
 void
 usage ()
 {
   printf (_("\
-Proxyknife %s, a customizable proxy hunter.\n\
+GNU Proxyknife %s, a customizable proxy hunter.\n\
 Usage: proxyknife [OPTION]...\n\
 Special options:\n\
 -V,  --version    display the version of Proxyknife and exit.\n\
      --help       print this help.\n\
 \n\
-All of the other options need arguments except '-d'!\n\
+All of the following options need arguments if no declarations.\n\
 \n\
 Commandline only options:\n\
--f \tthe argument is configuration file\n\
+-f \tthe argument is configuration file.\n\
 \n\
 Configuration options:\n\
 -d \tdebug, turn on debug output.\n\
-   \tNotice: no argument for this option\n\
+   \tNotice: no argument for this option.\n\
 -t \tthreads\n\
 -g \tuser_agent\n\
+-I \tproxyknifeintype, the type of proxylist(local file or url).\n\
+-i \tproxyknifein, the proxylist file or url used as input of proxyknife.\n\
+   \t'-' meanings stdin.\n\
 -m \tmytype, the type of myproxy.\n\
+-h \tmyhost, the IPV4 address or host name of myproxy.\n\
 -p \tmyport, the port of myproxy.\n\
--h \tmyhost, the ip or domainname of myproxy.\n\
 -u \tmyuser, the username of myproxy if need.\n\
 -s \tmypass, the password of myproxy if need.\n\
--c \tchecksite, the target site used to check testproxies.\n\
+-c \tchecksite, the target site used to check testproxies. Google by default.\n\
+-C \tcheckreqin, the file including req string.\n\
 -k \tkey, the keyword used to check whether testproxies give correct response.\n\
 -r \trcvtimeo, socket recv timeout in seconds.\n\
 -n \tsndtimeo, socket send timeout in seconds.\n\
@@ -72,13 +78,13 @@ Configuration options:\n\
 -T \ttestconreplysize\n\
 -G \ttesttagreplysize\n\
 -H \thttptestmethod\n\
--5 \tsocks5testmethod\n\
--4 \tsocks4testmethod\n\
+-5 \tsocks5testmethod, reserved now\n\
+-4 \tsocks4testmethod, reserved now\n\
 -l \tlogfilename\n\
--I \tproxyknifeintype, the type of proxylist(local file or url)\n\
--i \tproxyknifein, the proxylist file or url used as input of proxyknife.\n\
-   \t'-' meanings stdin.\n\
--o \tproxyknifeout, the file which proxyknife output result to. \n"), VERSION);
+-o \tproxyknifeout, the result is also printed to this file.\n\
+\n\
+Mail Bug reports and suggestions to <bug-proxyknife@gnu.org>.\n"), 
+VERSION);
 }
 
 /* Read file PROXYKNIFE_CONF and configure global vars */
@@ -94,29 +100,33 @@ READCONF (int argc, char **argv)
   ssize_t read;
   char *offset;
   char *p, *q;
-  /* must be customized */
+  /* must be customized. Have default guess. */
   int set_mytype = 0, set_httptestmethod = 0, set_socks5testmethod =
     0, set_socks4testmethod = 0;
-  /* useful only if mytype != DIRECT */
+  /* Must too only if mytype != DIRECT. Without default value */
   int set_myport = 0, set_myhost = 0;
-  /* userful only if mytype = xx_AUTH */
+  /* Must too only if mytype = *_AUTH. Without default value */
   int set_myuser = 0, set_mypass = 0;
   /* The following is customizable and  is also ok to use the value in 
      example. But do not remove or comment any of the following.
    */
-  int set_threads = 0, set_checksite = 0, set_key = 0, set_logfilename = 0;
+  /* These with hard-coded value by default. Must have value. */
+  int set_proxyknifein = 0, set_proxyknifeout = 0, set_proxyknifeintype = 0;
+  int set_debug = 0; 
+  
+  int set_threads = 0;
+  int set_checksite = 0, set_key = 0, set_logfilename = 0;
   int set_user_agent = 0, set_myconreplysize = 0, set_testconreplysize = 0;
   int set_testtagreplysize = 0, set_rcvtimeo = 0, set_sndtimeo = 0;
 
-  /* These with hard-coded policy. */
-  int set_proxyknifein = 0, set_proxyknifeout = 0, set_proxyknifeintype = 0;
-  int set_debug = 0;
+  /* Full customized. It is ok whether it exists. */
+  int set_checkreqin = 0;
 
   struct hostent *h;
 
   int slash, colon;
-  unsigned char *path, *home;
-  int pathlen;
+  unsigned char *filename, *home;
+  int filenamelen;
 
 
   /* for command option */
@@ -130,12 +140,39 @@ READCONF (int argc, char **argv)
     {"version", 0, NULL, 'V'},
     {0, 0, 0, 0}
   };
+  
+  /* Default value */
+  /* Guess. User must consider these. */
+  my.mytype = DIRECT;
+  test.httptestmethod = HTTP_GET;
+
+  /* Guess. But now only support the default: */
+  test.socks5testmethod = SOCKS5_CONNECT;
+  test.socks4testmethod = SOCKS4_CONNECT;
+
+
+  /* Default value. No change is also ok. */
+  threads = THREADS;
+  
+  target.target = xstrdup(CHECKSITE);
+  target.key = xstrdup(KEY);
+  
+  logfilename = xstrdup(LOGFILENAME);
+  
+  user_agent = xstrdup(USER_AGENT);
+
+  my.myconreplysize = MYCONREPLYSIZE;
+  test.testconreplysize = TESTCONREPLYSIZE;
+  test.testtagreplysize = TESTTAGREPLYSIZE;
+  rcvtimeo = RCVTIMEO;
+  sndtimeo = SNDTIMEO;
+  
 
 
   while (1)
     {
       c = getopt_long (argc, argv,
-		       "Vdf:i:I:o:t:g:m:p:h:u:s:c:k:r:n:S:T:G:H:5:4:l:",
+		       "Vdf:i:I:o:t:g:m:p:h:u:s:c:C:k:r:n:S:T:G:H:5:4:l:",
 		       long_options, &option_index);
       if (c == -1)
 	break;
@@ -156,7 +193,7 @@ READCONF (int argc, char **argv)
 	  break;
 	case 'V':
 	  printf (_("\
-Proxyknife %s\n\
+GNU Proxyknife %s\n\
 \n\
 Copyright (C) 2005-2006 %s <skyroam@gmail.com>.\n\
 Proxyknife is distributed in the hope that it will be useful,\n\
@@ -197,6 +234,9 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	case 'c':		/* checksite */
 	  optstr (optarg, &(target.target), &set_checksite);
 	  break;
+	case 'C':		/* "checkreqin=" */
+	  optstr (optarg, &(target.checkreqin), &set_checkreqin);
+	  break;
 	case 'k':		/* "key=" */
 	  optstr (optarg, &(target.key), &set_key);
 	  break;
@@ -235,7 +275,9 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	}
     }
 
-  conf = fopen (proxyknife_conf, "r");
+  if(proxyknife_conf!=NULL)conf = fopen (proxyknife_conf, "r");
+  else conf = NULL;
+
   if (conf == NULL)
     {
       if (debug)
@@ -251,12 +293,12 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	  home = getenv ("HOME");
 	  if (home != NULL)
 	    {
-	      pathlen =
+	      filenamelen =
 		strlen (home) + strlen (PROXYKNIFE_CONF) + strlen ("/.");
-	      path = xmalloc (pathlen + 1);
-	      memset (path, 0, pathlen);
-	      snprintf (path, pathlen + 1, "%s/.%s", home, PROXYKNIFE_CONF);
-	      conf = fopen (path, "r");
+	      filename = xmalloc (filenamelen + 1);
+	      memset (filename, 0, filenamelen);
+	      snprintf (filename, filenamelen + 1, "%s/.%s", home, PROXYKNIFE_CONF);
+	      conf = fopen (filename, "r");
 	      if (conf == NULL)
 		{
 		  if (debug)
@@ -267,7 +309,7 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 		  if (debug)
 		    printf ("Using ~/." PROXYKNIFE_CONF "\n");
 		}
-	      xfree ((void **) &path);
+	      xfree ((void **) &filename);
 	    }
 	}
       else
@@ -280,25 +322,23 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
       printf ("READCONF:Using %s\n", proxyknife_conf);
     }
 
-  xfree ((void **) &proxyknife_conf);
+  if(proxyknife_conf!=NULL)xfree ((void **) &proxyknife_conf);
   /* check  /etc/proxyknife.conf */
   /* error  was deal above, 
      so there will not be  duplicate reports of errors  here */
   if (conf == NULL)
     {
-      /* pathlen = strlen(PROXYKNIFE_CONF)+strlen("/etc/");
-         path = xmalloc(pathlen);
-         memset(path,0,pathlen);
-         snprintf(path,pathlen,"/etc/%s",PROXYKNIFE_CONF);
-       */
-      path = SYSTEM_PROXYKNIFECONF;	/* so xfree is unnecessary... */
-      conf = fopen (path, "r");
+      filename = SYSTEM_PROXYKNIFECONF;	/* so xfree is unnecessary... */
+      conf = fopen (filename, "r");
       if (conf == NULL)
 	{
-	  perror ("READCONF:fopen:/etc/" PROXYKNIFE_CONF);
-	  fprintf (stderr,
-		   "READCONF:FATAL ERROS: Can't find proxyknife.conf");
-	  xexit (EXIT_FAILURE);
+	  if(debug){
+              perror ("READCONF:fopen:/etc/" PROXYKNIFE_CONF);
+	  fprintf (stderr, 
+		   "READCONF:Can't find the configuration file!Use commadline options only!\n");
+      }
+
+	  /* xexit (EXIT_FAILURE); */
 	}
       else
 	{
@@ -310,7 +350,7 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
   slash = '/';
   colon = ':';
 
-
+  if( conf!= NULL){
   while (1)
     {
       line = NULL;
@@ -320,8 +360,10 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	break;
       if (read > 0)
 	{			/*read=0?  */
-	  if (line[0] == '#')
-	    continue;
+	  if (line[0] == '#'){
+              xfree((void **)&line);
+              continue;
+      }
 	  if (line[read - 1] == '\n')
 	    line[read - 1] = 0;
 	  !readint (line, "threads=", &threads, &set_threads)
@@ -332,6 +374,8 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	    || !readstr (line, "myuser=", &(my.myuser), &set_myuser)
 	    || !readstr (line, "mypass=", &(my.mypass), &set_mypass)
 	    || !readstr (line, "checksite=", &(target.target), &set_checksite)
+	    || !readstr (line, "checkreqin=", &(target.checkreqin),
+			 &set_checkreqin)
 	    || !readstr (line, "key=", &(target.key), &set_key)
 	    || !readint (line, "rcvtimeo=", &(rcvtimeo), &set_rcvtimeo)
 	    || !readint (line, "sndtimeo=", &(sndtimeo), &set_sndtimeo)
@@ -358,24 +402,25 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	    || !readint (line, "debug=", &debug, &set_debug);
 	}
       if (line)
-	free (line);
+	xfree ((void **)&line);
     }
 
-  if (conf)
-    {
       if (fclose (conf) != 0)
 	{
-	  perror ("READCONF:fclose:" PROXYKNIFE_CONF);
+	  perror (__FILE__ ": READCONF:fclose: configuration file" );
 	  xexit (EXIT_FAILURE);
 	}
+
     }
-  if (set_mytype * set_httptestmethod * set_socks5testmethod * set_rcvtimeo
-      * set_sndtimeo * set_checksite * set_threads * set_key * set_user_agent
+  if (set_mytype * set_httptestmethod * set_socks5testmethod  
+      * set_socks4testmethod * set_rcvtimeo * set_sndtimeo 
+      * set_checksite * set_key * set_user_agent * set_threads  
       * set_myconreplysize * set_testconreplysize * set_testtagreplysize
-      * set_logfilename == 0)
-    {
-      fprintf (stderr, "%s: READCONF:too few config parameters:", progname);
-      fprintf (stderr, "set flag are checked:\n"
+      * set_logfilename
+     )
+    {if(debug){
+      fprintf (stderr, "%s: READCONF:not all parameters reset:", progname);
+      fprintf (stderr, "set flags are checked:\n"
 	       "\tmytype:%d\n"
 	       "\thttptestmethod:%d\n"
 	       "\tsocks5testmethod:%d\n"
@@ -383,9 +428,9 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	       "\trcvtimeo:%d\n"
 	       "\tsndtimeo:%d\n"
 	       "\tchecksite:%d\n"
-	       "\tthreads:%d\n"
-	       "\tkey:%d\n"
-	       "\tuser_agent:%d\n"
+           "\tkey:%d\n"
+           "\tuser_agent:%d\n"
+           "\tthreads:%d\n"
 	       "\tmyconreplysize:%d\n"
 	       "\ttestconreplysize:%d\n"
 	       "\ttesttagreplysize:%d\n"
@@ -393,10 +438,11 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	       set_mytype, set_httptestmethod, set_socks5testmethod,
 	       set_socks4testmethod,
 	       set_rcvtimeo, set_sndtimeo,
-	       set_checksite, set_threads, set_key, set_user_agent,
+	       set_checksite, set_key,set_user_agent,set_threads,
 	       set_myconreplysize, set_testconreplysize,
-	       set_testtagreplysize, set_logfilename);
-      xexit (EXIT_FAILURE);
+	       set_testtagreplysize,set_logfilename );
+              }
+      /* xexit (EXIT_FAILURE); */
     }
 
   if (my.mytype != DIRECT)
@@ -412,7 +458,7 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
 	      xexit (EXIT_FAILURE);
 	    }
 	}
-      else
+      else if ((my.mytype != HTTP_CONNECT) && (my.mytype != SOCKS5_CONNECT))
 	{
 	  fprintf (stderr,
 		   "%s: READCONF: Parameters:mytype %d is not supported now!\n",
@@ -438,19 +484,36 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
     }
 
   /* http://$targethost:$targetport$targetpath */
+  /* or host:port[path] */
   /* Parse it here to stop proxyknife early and let user know the error in conf */
-  p = strstr ((const char *) target.target, (const char *) "http://");
+  /* modify to accept protocol:// or even no // */
+  p = strstr ((const char *) target.target, (const char *) "://");
   if (p == NULL)
+    {				/* haven't dup now! */
+      if (debug)
+	fprintf (stderr, "%s %s: READCONF: No '://' found in %s\n",
+		 progname, __FILE__, target.target);
+      //xexit (EXIT_FAILURE);
+      p = target.target;
+    }
+  else
     {
-      fprintf (stderr, "%s: READCONF:No http:// found in %s\n", progname,
-	       target.target);
-      xexit (EXIT_FAILURE);
+      stringlen = p - target.target;
+      if (stringlen != 0)
+	{			/* we do not accept NULL proto, so ignored it. */
+	  target.protocol = xmalloc (stringlen + 1);
+	  memmove (target.protocol, target.target, stringlen);
+	  target.protocol[stringlen] = '\0';
+	}
+
+      p += (int) strlen ("://");
     }
-  p += (int) strlen ("http://");
   q = strchr (p, colon);
   if (q == NULL)
     {
-      fprintf (stderr, "%s :No ':' found in %s\n", progname, p);
+      if (debug)
+	fprintf (stderr, "%s %s: READCONF: No ':' found in %s\n",
+		 progname, __FILE__, p);
       xexit (EXIT_FAILURE);
     }
   stringlen = q - p;
@@ -462,21 +525,31 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
   q = strchr (p, slash);
   if (q == NULL)
     {				/* add fix in the future */
-      fprintf (stderr, "%s: READCONF:No / found in %s\n", progname, p);
-      xexit (EXIT_FAILURE);
+      if (debug)
+	fprintf (stderr, "%s %s: READCONF: No '/' found in %s\n",
+		 progname, __FILE__, p);
+      //xexit (EXIT_FAILURE);
+      /* default to '/' */
+      target.targetpath = xmalloc (2);
+      target.targetpath[0] = '/';
+      target.targetpath[1] = '\0';
+      /* for port */
+      stringlen = strlen (p);
     }
-  stringlen = q - p;
+  else
+    {				/* path */
+      stringlen = strlen (q);
+      target.targetpath = xmalloc (stringlen + 1);
+      memmove (target.targetpath, q, stringlen);
+      target.targetpath[stringlen] = '\0';
+      /* for port */
+      stringlen = q - p;
+    }
+
   target.targetport = xmalloc (stringlen + 1);
   memmove (target.targetport, p, stringlen);
-  /* fprintf(stderr,"%d %s %s\n",stringlen,p,q); */
   target.targetport[stringlen] = '\0';
 
-  /* path */
-  stringlen = strlen (q);
-  target.targetpath = xmalloc (stringlen + 1);
-  memmove (target.targetpath, q, stringlen);
-  target.targetpath[stringlen] = '\0';
-
   /*sscanf ((const char *) target, "http://%[^:]:%[^/]%s", target_host,
      target_port, target_path); */
 
@@ -489,13 +562,13 @@ GNU General Public License for more details.\n"), VERSION, _("Jia Wang"));
     }
 
   /* init target.targetaddr,used to validate socks5 proxy */
+  /* copy it to sin_addr then we can use gethostbyname later */
   target.targetaddr.sin_addr = *(struct in_addr *) (h->h_addr);
   target.targetaddr.sin_port = htons (atoi (target.targetport));
   target.targetaddr.sin_family = AF_INET;
   memset (&(target.targetaddr.sin_zero), 0,
 	  sizeof (target.targetaddr.sin_zero));
 
-
   /* proxyknife_in, proxyknife_out */
 
   /* If no NULL options or NULL configuration lines */
@@ -642,6 +715,22 @@ readstr (char *line, char *flagstring, char **s, int *num)
   return -1;
 }
 
+/* Make a duplicate of a string with allocated dynamic memory */
+
+void *
+xstrdup(char *s)
+{
+        int len;
+        char *p;
+        assert(s!=NULL);
+        len = strlen(s);
+        assert(len!=0);
+        len ++;
+        p = xmalloc(len);
+        memmove(p,s,len);
+        return p;
+}
+
 /* read a string option from commandline */
 void
 optstr (char *value, char **s, int *num)
@@ -651,14 +740,16 @@ optstr (char *value, char **s, int *num)
   /* check  duplication */
   if (*num)
     {
-      fprintf (stderr, "Duplication commandline options: %s!\n", value);
+      fprintf (stderr, __FILE__
+	       ": Duplication commandline options: %s!\n", value);
       exit (EXIT_FAILURE);
     }
   else
     {
+      assert (value != NULL);	/* This should not happen. */
       //xfree(s);
       len = strlen (value) + 1;
-      *s = xmalloc (len);
+      *s = xmalloc (len); /* *s is unallocated before this. */
       memmove (*s, value, len);
       (*num)++;
     }
