@@ -1,22 +1,24 @@
-/*  socks4.c - routines used to interact with socks5 proxy
-    Copyright (C) 2005-2006  Jia Wang (skyroam@gmail.com) 
-    
-    This file is part of Proxyknife.
-
-    Proxyknife is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Proxyknife is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Foobar; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-*/
+/*  socks4.c - routines used to interact with socks4 proxy
+ * Copyright (C) 2005-2006 Jia Wang <skyroam@gmail.com>
+ *
+ *
+ *
+ * This file is part of GNU Proxyknife.
+ * GNU Proxyknife is free software; you can redistribute it and/or modify it 
+ * under the terms of the GNU General Public License as published by the 
+ * Free Software Foundation; either version 2 of the License, or (at your 
+ * option) any later version.
+ *
+ * GNU Proxyknife is distributed in the hope that it will be useful, 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ * General Public License for more details.
+ *
+ *
+ * You should have received a copy of the GNU General Public License 
+ * along with Proxyknife; if not, write to the Free Software 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -64,21 +66,57 @@ s4con (int sockfd, struct sockaddr *serv)
 	  if ((p[0] == 0) && (p[1] == 90))
 	    return 0;
 	  else
-	    fprintf (stderr, "SOCKS4 server error reply:%02X %02X\n",
-		     p[0], p[1]);
+	    fprintf (stderr,
+		     "%s %s: s4con:SOCKS4 server error reply:%02X %02X\n",
+		     progname, __FILE__, p[0], p[1]);
 	}
     }
   return -1;
 }
 
-/* Unfinished now */
+int
+CONVIASOCKS4_MAIN (int sockfd, struct sockaddr *addr)
+{
+  int ret, len, offset;
+  char *p;
+
+  ret =
+    connect (sockfd, (struct sockaddr *) &(my.myaddr),
+	     sizeof (struct sockaddr));
+  if (ret == -1)
+    {
+      /* errno is thread-safety */
+      /* perror ("CONVIASOCKS4:connect:Connect to myaddr"); */
+      fprintf (stderr, "%s %s: CONVIASOCKS4_MAIN:connect:Connect to myaddr\n",
+	       progname, __FILE__);
+      return -1;
+    }
+
+  /* build request */
+  if (my.mytype != SOCKS4_CONNECT)
+    {
+      fprintf (stderr, "%s %s: "
+	       "CONVIASOCKS4_MAIN:error method!This message shouldn't be printed!\n",
+	       progname, __FILE__);
+      abort ();
+      /* return -1; */
+    }
+
+  /* notice: user may be customized for the need of identd */
+  /* send */
+  ret = s4con (sockfd, (struct sockaddr *) addr);
+  if (ret == -1)
+    return -1;
+  return 0;
+}
+
+
 /* ***************************inside thread **********************/
 
-/* Build connection to remote server through free or username/password-
- * authentication socks5 proxy. 
+/* Build connection to remote server through free socks4 proxy. 
  *
- * The time sending connect command to socks5 proxy is stored in *start.
- * The time receiving success  reply from socks5 proxy is stored in * end.
+ * The time sending connect command to socks4 proxy is stored in *start.
+ * The time receiving successful reply from socks4 proxy is stored in *end.
  * 
  * Return value:
  * 0: success!
@@ -96,15 +134,19 @@ CONVIASOCKS4 (int sockfd, struct timeval *start,
 	     sizeof (struct sockaddr));
   if (ret == -1)
     {
-      perror ("CONVIASOCKS4:connect:Connect to myaddr");
+      /* errno is thread-safety */
+      /* perror ("CONVIASOCKS4:connect:Connect to myaddr"); */
+      fprintf (stderr, "%s %s: CONVIASOCKS4:connect:Connect to myaddr\n",
+	       progname, __FILE__);
       return -1;
     }
 
   /* build request */
   if (my.mytype != SOCKS4_CONNECT)
     {
-      fprintf (stderr, "%s: " __FILE__
-	       " CONVIASOCKS4:This message can't  be printed!\n", progname);
+      fprintf (stderr, "%s %s: "
+	       " CONVIASOCKS4:error method!This message shouldn't be printed!\n",
+	       progname, __FILE__);
       abort ();
       /* return -1; */
     }
