@@ -1,22 +1,24 @@
-/*  socks5.c - routines used to interact with socks5 proxy
-    Copyright (C) 2005-2006  Jia Wang (skyroam@gmail.com) 
-    
-    This file is part of Proxyknife.
-
-    Proxyknife is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Proxyknife is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Foobar; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-*/
+/* socks5.c - routines used to interact with socks5 proxy
+ * Copyright (C) 2005-2006 Jia Wang <skyroam@gmail.com>
+ *
+ *
+ *
+ * This file is part of GNU Proxyknife.
+ * GNU Proxyknife is free software; you can redistribute it and/or modify it 
+ * under the terms of the GNU General Public License as published by the 
+ * Free Software Foundation; either version 2 of the License, or (at your 
+ * option) any later version.
+ *
+ * GNU Proxyknife is distributed in the hope that it will be useful, 
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ * General Public License for more details.
+ *
+ *
+ * You should have received a copy of the GNU General Public License 
+ * along with Proxyknife; if not, write to the Free Software 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -159,7 +161,7 @@ CONVIASOCKS5_MAIN (int sockfd, struct sockaddr *addr)
 	     sizeof (struct sockaddr));
   if (ret == -1)
     {
-      perror ("CONVIASOCKS5:connect:Connect to myaddr");
+      perror (__FILE__ ": CONVIASOCKS5_MAIN:connect:Connect to myaddr");
       return -1;
     }
 
@@ -179,8 +181,9 @@ CONVIASOCKS5_MAIN (int sockfd, struct sockaddr *addr)
     }
   else
     {
-      fprintf (stderr, "%s: CONVIASOCKS5:This message can't  be printed!\n",
-	       progname);
+      fprintf (stderr,
+	       "%s %s: CONVIASOCKS5_MAIN:This message can't  be printed!\n",
+	       progname, __FILE__);
       abort ();
       /* return -1; */
     }
@@ -216,7 +219,8 @@ CONVIASOCKS5 (int sockfd, struct timeval *start,
 	     sizeof (struct sockaddr));
   if (ret == -1)
     {
-      perror ("CONVIASOCKS5:connect:Connect to myaddr");
+      fprintf (stderr, "%s %s: CONVIASOCKS5:connect:Connect to myaddr",
+	       progname, __FILE__);
       return -1;
     }
 
